% Conformally map a genus-0 point cloud to the unit sphere
% 
% Usage:
% map = pc_spherical_conformal_map(vertex)
% 
% Input:
% vertex: nv x 3 vertex coordinates of the input point cloud
%
% Output:
% map: nv x 3 vertex coordinates of the spherical conformal parameterization
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P.-T. Choi, K. T. Ho and L. M. Lui, 
%     "Spherical Conformal Parameterization of Genus-0 Point Clouds for Meshing."
%     SIAM Journal on Imaging Sciences, 9(4), pp. 1582-1618, 2016.
% 
% Copyright (c) 2015-2018, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

addpath('mfile')
addpath('spherical_delaunay')

%% load point cloud data
load('david.mat');

figure; 
plot3(vertex(:,1),vertex(:,2),vertex(:,3),'r.'); 
axis equal tight
title('The input point cloud')
view([-140 10])

%% Spherical conformal parameterization
tic;
map = pc_spherical_conformal_map(vertex);
toc;

figure;
plot3(map(:,1),map(:,2),map(:,3),'r.'); 
axis equal tight
title('Spherical conformal parameterization')

%% To mesh the spherical point cloud and hence the input point cloud
% Spherical Delaunay triangulation
% Reference: https://people.sc.fsu.edu/~jburkardt/m_src/sphere_delaunay/sphere_delaunay.html
[~,face] = sphere_delaunay([],map');
face = face';

% Visualizing the meshing result
plot_mesh(map,face);
title('Delaunay triangulation on the spherical point cloud');

plot_mesh(vertex,face);
title('The induced triangulation on the original point cloud');
view([-140 10])

% Conformality distortion measured w.r.t. the induced triangulation
distortion = angle_distortion(vertex,face,map); 