function [L, rings] = calc_pc_laplacian(vertex, k)
% Compute the point cloud Lapalacian.
% 
% If you use this code in your own work, please cite the following paper:
% [1] G. P.-T. Choi, K. T. Ho and L. M. Lui, 
%     "Spherical Conformal Parameterization of Genus-0 Point Clouds for Meshing."
%     SIAM Journal on Imaging Sciences, 9(4), pp. 1582-1618, 2016.
% 
% Copyright (c) 2015-2018, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

numofv = size(vertex,1);

rings = cell(numofv,1);

if size(vertex,2)==2
    vertex = [vertex, zeros(numofv,1)];
end

knnInd = knnsearch(vertex, vertex, 'K', k);

[coeff, powerc] = calc_coeff_power(2, 0, 0);

v_local_1 = reshape(vertex(knnInd,1), [numofv,k]);
v_local_2 = reshape(vertex(knnInd,2), [numofv,k]);
v_local_3 = reshape(vertex(knnInd,3), [numofv,k]);

M = zeros(k,numofv);
N = zeros(k,numofv);
val = zeros(k,numofv);

for j=1:numofv
        knn = zeros(k,3);
        % center to v(j,:)
        knn(:,1) = v_local_1(j,:)' - vertex(j,1);
        knn(:,2) = v_local_2(j,:)' - vertex(j,2);
        knn(:,3) = v_local_3(j,:)' - vertex(j,3);
        
        PCA = pca(knn,'Rows','complete');
        
        knn_local = knn * PCA;
        proj_k = knn - knn_local(:,3) * PCA(:,3)';
        knn_local(:,1:2) = proj_k * PCA(:,1:2);
        
        x_ = knn_local(:,1);
        y_ = knn_local(:,2);
    
        M(:,j) = j;
        N(:,j) = knnInd(j,:)';
    
        norm = x_.^2+ y_.^2;
        maxnorm = max(norm);
        
        weight = exp(-sqrt(k)*norm/maxnorm)/k;

        weight(1) = 1;
        
        A = zeros(length(coeff), length(coeff));
        B = zeros(length(coeff), k);
        for i=1:k
            b = coeff .* (x_(i).^powerc(:,1)) .* (y_(i).^powerc(:,2));
            B(:,i) = weight(i) * b;
            A = A + B(:,i) * b';
        end
        
        c = A\B;
        
        bx_ = [0;1;0;0;0;0];
        by_ = [0;0;1;0;0;0];
        bxx_ = [0;0;0;2;0;0];
        bxy_ = [0;0;0;0;1;0];
        byy_ = [0;0;0;0;0;2];
        D = [bx_, by_, bxx_, bxy_, byy_].' * c * knn_local(:,3);
        
        val(:,j) = c'*...
            (bx_*(2*D(1)^2*D(2)*D(4)-D(1)*D(3)-D(1)*D(2)^2*D(3)-D(1)*D(5)-D(1)^3*D(5))...
            +by_*(2*D(1)*D(2)^2*D(4)-D(2)*D(3)-D(2)^3*D(3)-D(2)*D(5)-D(1)^2*D(2)*D(5))...
            +bxx_*((1+D(1)^2+D(2)^2)*(1+D(2)^2))...
            +bxy_*((1+D(1)^2+D(2)^2)*(-2*D(1)*D(2)))...
            +byy_*((1+D(1)^2+D(2)^2)*(1+D(1)^2)));
        
        
        KNNindex_j = knnInd(j,:);
        
        TRI = delaunay(x_,y_);

        [local_mesh_ind, ~] = find(KNNindex_j(TRI)==j);
        rings{j}=KNNindex_j(TRI(local_mesh_ind,:));
end

L = sparse(M, N, val, numofv, numofv);
end