function map = pc_spherical_conformal_map(vertex)
% Conformally map a genus-0 point cloud to the unit sphere
% 
% Usage:
% map = pc_spherical_conformal_map(vertex)
% 
% Input:
% vertex: nv x 3 vertex coordinates of the input point cloud
%
% Output:
% map: nv x 3 vertex coordinates of the spherical conformal parameterization
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P.-T. Choi, K. T. Ho and L. M. Lui, 
%     "Spherical Conformal Parameterization of Genus-0 Point Clouds for Meshing."
%     SIAM Journal on Imaging Sciences, 9(4), pp. 1582-1618, 2016.
% 
% Copyright (c) 2015-2018, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

%% Compute laplacian
[L, rings] = calc_pc_laplacian(vertex, 25);

%% Find the most regular triangle in local meshes
numofv = length(vertex);
L = L(1:numofv,1:numofv);

face_temp = cell2mat(rings);

temp = vertex(reshape(face_temp',1,length(face_temp)*3),1:3);
e1 = sqrt(sum((temp(2:3:end,1:3) - temp(3:3:end,1:3))'.^2))';
e2 = sqrt(sum((temp(1:3:end,1:3) - temp(3:3:end,1:3))'.^2))';
e3 = sqrt(sum((temp(1:3:end,1:3) - temp(2:3:end,1:3))'.^2))';
regularity = abs(e1./(e1+e2+e3)-1/3)+...
    abs(e2./(e1+e2+e3)-1/3)+abs(e3./(e1+e2+e3)-1/3);
[~,bigtri] = min(regularity);
bd = face_temp(bigtri,:);

%% North pole iteration
v1 = vertex(bd(1),:);
v2 = vertex(bd(2),:);
v3 = vertex(bd(3),:);

l12 = norm(v1-v2);
l23 = norm(v2-v3);
l31 = norm(v1-v3);

angle2 = (l12^2+l23^2-l31^2)/2/l12/l23;

d = vertex(:,1)*0;
d(bd) = [0 l12 -l23*exp(1i*acos(angle2))+l12]/l12*2.6562^2;
d(bd) = d(bd) - mean(d(bd));

[ii,jj,kk] = find(L(bd,:));
L2 = L - sparse(bd(ii),jj,kk,numofv,numofv) + sparse(bd,bd,[1 1 1],numofv,numofv);
z = L2\d;
z = z - mean(z);

map = [2*real(z)./(1+real(z).^2+imag(z).^2), 2*imag(z)./(1+real(z).^2+imag(z).^2), (-1+real(z).^2+imag(z).^2)./(1+real(z).^2+imag(z).^2)];

%% South pole iteration
ratio = 0.2;

[~, index] = sort(map(:,3));
fixed = index(1:floor(length(vertex)*ratio));

w = complex(map(:,1)./(1+map(:,3)) , map(:,2)./(1+map(:,3)));
d = w*0; d(fixed) = w(fixed);

[ii,jj,kk] = find(L(fixed,:));
L2 = L - sparse(fixed(ii),jj,kk,numofv,numofv) + sparse(fixed,fixed,ones(length(fixed),1),numofv,numofv);
w = L2\d;

map = [2*real(w)./(1+abs(w).^2), 2*imag(w)./(1+abs(w).^2), -(abs(w).^2-1)./(1+abs(w).^2)];

%% balance the distribution of one ring at poles
[~,id] = sort(map(:,3),'descend');
north = id(1);
south = id(end);

north_ring = setdiff(unique((rings{north})),north);
south_ring = setdiff(unique((rings{south})),south);

z = complex(map(:,1)./(1-map(:,3)) , map(:,2)./(1-map(:,3)));
w = complex(map(:,1)./(1+map(:,3)) , map(:,2)./(1+map(:,3)));
NorthTriSide = mean(abs(z(north_ring)-z(north)));
SouthTriSide = mean(abs(w(south_ring)-w(south)));

w = w*(sqrt(NorthTriSide*SouthTriSide))/(SouthTriSide); % rescale to get the best distribution

map = [2*real(w)./(1+abs(w).^2), 2*imag(w)./(1+abs(w).^2), -(abs(w).^2-1)./(1+abs(w).^2)];

%% N_S reiteration
map_temp = map;
reiterate = 1;
difference_prev = Inf;

while reiterate == 1
    % N re-iterate
    [~, index] = sort(map(:,3),'descend');
    fixed = index(1:floor(length(vertex)*ratio));

    z = complex(map(:,1)./(1-map(:,3)) , map(:,2)./(1-map(:,3)));
    d = z*0; d(fixed) = z(fixed);

    [ii,jj,kk] = find(L(fixed,:));
    L2 = L - sparse(fixed(ii),jj,kk,numofv,numofv)+sparse(fixed,fixed,ones(length(fixed),1),numofv,numofv);
    z = L2\d;

    map = [2*real(z)./(1+real(z).^2+imag(z).^2),2*imag(z)./(1+real(z).^2+imag(z).^2),(-1+real(z).^2+imag(z).^2)./(1+real(z).^2+imag(z).^2)];
      
    % S re-iterate
    [~, index] = sort(map(:,3),'ascend');
    fixed = index(1:floor(length(vertex)*ratio));

    w = complex(map(:,1)./(1+map(:,3)) , map(:,2)./(1+map(:,3)));
    d = w*0; d(fixed) = w(fixed);

    [ii,jj,kk] = find(L(fixed,:));
    L2 = L - sparse(fixed(ii),jj,kk,numofv,numofv) + sparse(fixed,fixed,ones(length(fixed),1),numofv,numofv);
    w = L2\d;

    map = [2*real(w)./(1+abs(w).^2), 2*imag(w)./(1+abs(w).^2), -(abs(w).^2-1)./(1+abs(w).^2)];
    
    % checking
    difference = map - map_temp;
    map_temp = map;
    difference = mean((sum(difference.^2,2)));
    if difference < 0.001 || difference > difference_prev
        reiterate = 0;
    else
        difference_prev = difference;
    end

end

%% balance the distribution of one ring at poles
[~,id] = sort(map(:,3),'descend');
north = id(1);
south = id(end);

north_ring = setdiff(unique((rings{north})),north);
south_ring = setdiff(unique((rings{south})),south);

z = complex(map(:,1)./(1-map(:,3)) , map(:,2)./(1-map(:,3)));
w = complex(map(:,1)./(1+map(:,3)) , map(:,2)./(1+map(:,3)));
NorthTriSide = mean(abs(z(north_ring)-z(north)));
SouthTriSide = mean(abs(w(south_ring)-w(south)));

w = w*(sqrt(NorthTriSide*SouthTriSide))/(SouthTriSide); % rescale to get the best distribution

map = [2*real(w)./(1+abs(w).^2), 2*imag(w)./(1+abs(w).^2), -(abs(w).^2-1)./(1+abs(w).^2)];

end
